//+---------------------------------------------------------------------+
//|                                                             WPR.mq5 | 
//|                                Copyright  2011,   Nikolay Kositsin | 
//|                                 Khabarovsk,   farria@mail.redcom.ru | 
//+---------------------------------------------------------------------+
//|     SmoothAlgorithms.mqh      |
//|  : terminal_data_folder\MQL5\Include                          |
//+---------------------------------------------------------------------+
#property copyright "Copyright  2011, Nikolay Kositsin"
#property link "farria@mail.redcom.ru"
#property description "Larry Williams' Percent Range"
//----   
#property version   "1.00"
//----     
#property indicator_separate_window
//----   
#property indicator_buffers 4 
//----  2  
#property indicator_plots   2
//+-----------------------------------+
//|     1 |
//+-----------------------------------+
//----      
#property indicator_type1   DRAW_COLOR_LINE
//----      
#property indicator_color1 Gray,LightSeaGreen,HotPink
//----   - 
#property indicator_style1  STYLE_SOLID
//----     4
#property indicator_width1  4
//----   
#property indicator_label1  "Signal"
//+-----------------------------------+
//|     2 |
//+-----------------------------------+
//----     
#property indicator_type2   DRAW_LINE
//----     
#property indicator_color2 Blue
//----   -  
#property indicator_style2  STYLE_SOLID
//----     2
#property indicator_width2  2
//----   
#property indicator_label2  "WPR"
//+----------------------------------------------+
//|     |
//+----------------------------------------------+
#property indicator_level1     -20.0
#property indicator_level2     -80.0
#property indicator_levelcolor Violet
#property indicator_levelstyle STYLE_DASHDOTDOT
//+-----------------------------------+
//|    CXMA             |
//+-----------------------------------+
#include <SmoothAlgorithms.mqh> 
//+-----------------------------------+
//----    CXMA   SmoothAlgorithms.mqh
CXMA XSIGN;
//+-----------------------------------+
//|             |
//+-----------------------------------+
enum Applied_price_ //  
  {
   PRICE_CLOSE_ = 1,     // Close
   PRICE_OPEN_,          // Open
   PRICE_HIGH_,          // High
   PRICE_LOW_,           // Low
   PRICE_MEDIAN_,        // Median Price (HL/2)
   PRICE_TYPICAL_,       // Typical Price (HLC/3)
   PRICE_WEIGHTED_,      // Weighted Close (HLCC/4)
   PRICE_SIMPLE,         // Simple Price (OC/2)
   PRICE_QUARTER_,       // Quarted Price (HLOC/4) 
   PRICE_TRENDFOLLOW0_,  // TrendFollow_1 Price 
   PRICE_TRENDFOLLOW1_   // TrendFollow_2 Price 
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
enum IndStyle //   
  {
   COLOR_LINE = DRAW_COLOR_LINE,  //  
   COLOR_ARROW=DRAW_COLOR_ARROW   //  
  };
/*enum Smooth_Method -    SmoothAlgorithms.mqh
  {
   MODE_SMA_,  // SMA
   MODE_EMA_,  // EMA
   MODE_SMMA_, // SMMA
   MODE_LWMA_, // LWMA
   MODE_JJMA,  // JJMA
   MODE_JurX,  // JurX
   MODE_ParMA, // ParMA
   MODE_T3,    // T3
   MODE_VIDYA, // VIDYA
   MODE_AMA,   // AMA
  }; */
//+-----------------------------------+
//|         |
//+-----------------------------------+
input int DPeriod=15;                        //  WPR
input Smooth_Method SSmoothMethod=MODE_JJMA; //    
input int SPeriod=7;                         //   
input int SPhase=100;                        //   
input Applied_price_ IPC=PRICE_CLOSE;        //  
input int Shift=0;                           //      
input IndStyle Style=DRAW_COLOR_ARROW;       //   WPR
//+-----------------------------------+
//----   ,   
//----      
double WPR[],XWPR[];
double ColorXWPR[];
//----      
int StartBars,StartBarsD,StartBarsS;
//+------------------------------------------------------------------+   
//| WPR indicator initialization function                            | 
//+------------------------------------------------------------------+ 
void OnInit()
  {
//----     
   StartBarsD=DPeriod+1;
   StartBarsS=StartBarsD+XSIGN.GetStartBars(SSmoothMethod,SPeriod,SPhase);
   StartBars=StartBarsS;

//----       
   XSIGN.XMALengthCheck("SPeriod",SPeriod);
//----       
   XSIGN.XMAPhaseCheck("SPhase",SPhase,SSmoothMethod);

//----    XWPR[]   
   SetIndexBuffer(0,XWPR,INDICATOR_DATA);
//----    2  
   PlotIndexSetInteger(0,PLOT_SHIFT,Shift);
//----      
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,StartBars);
//----   ,      
   PlotIndexSetDouble(0,PLOT_EMPTY_VALUE,EMPTY_VALUE);

//----    ColorXWPR[]       
   SetIndexBuffer(1,ColorXWPR,INDICATOR_COLOR_INDEX);
//----      
   PlotIndexSetInteger(1,PLOT_DRAW_BEGIN,StartBars+1);

//----    WPR[]   
   SetIndexBuffer(2,WPR,INDICATOR_DATA);
//----    1  
   PlotIndexSetInteger(2,PLOT_SHIFT,Shift);
//----      
   PlotIndexSetInteger(2,PLOT_DRAW_BEGIN,StartBars);
//----   ,      
   PlotIndexSetDouble(2,PLOT_EMPTY_VALUE,EMPTY_VALUE);
//----       
   PlotIndexSetInteger(2,PLOT_DRAW_TYPE,Style);

//----      
   string shortname,Smooth;
   Smooth=XSIGN.GetString_MA_Method(SSmoothMethod);
   StringConcatenate(shortname,"Larry Williams' Percent Range(",string(DPeriod),",",Smooth,")");
//----           
   IndicatorSetString(INDICATOR_SHORTNAME,shortname);

//---     
   IndicatorSetInteger(INDICATOR_DIGITS,0);
//----  
  }
//+------------------------------------------------------------------+ 
//| WPR iteration function                                          | 
//+------------------------------------------------------------------+ 
int OnCalculate(const int rates_total,    //       
                const int prev_calculated,//       
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//----       
   if(rates_total<StartBars) return(0);

//----       
   double HH,LL,price_;
//----        
   int first1,first2,first3,bar;

//----    first    
   if(prev_calculated>rates_total || prev_calculated<=0) //      
     {
      first1=StartBarsD; //      
      first2=StartBarsD+1;
      first3=StartBars+1;
     }
   else
     {
      first1=prev_calculated-1; //      
      first2=first1;
      first3=first1;
     }

//----    
   for(bar=first1; bar<rates_total && !IsStopped(); bar++)
     {
      //----   PriceSeries     price_
      price_=PriceSeries(IPC,bar,open,low,high,close);

      ArraySetAsSeries(low,true);
      ArraySetAsSeries(high,true);
      int rate_=rates_total-1-bar;
      LL=low[ArrayMinimum(low,rate_,DPeriod)];
      HH=high[ArrayMaximum(high,rate_,DPeriod)];

      //----      
      if(HH!=LL) WPR[bar]=-(HH-price_)*100/(HH-LL);
      else       WPR[bar]=WPR[bar-1];

      //----      
      XWPR[bar]=XSIGN.XMASeries(StartBarsD,prev_calculated,rates_total,SSmoothMethod,SPhase,SPeriod,WPR[bar],bar,false);
     }

//----     
   for(bar=first3; bar<rates_total; bar++)
     {
      ColorXWPR[bar]=0;
      if(XWPR[bar]>XWPR[bar-1]) ColorXWPR[bar]=1;
      if(XWPR[bar]<XWPR[bar-1]) ColorXWPR[bar]=2;
     }
//----     
   return(rates_total);
  }
//+------------------------------------------------------------------+
